@@ -5,16 +5,14 @@ mod record;
 
 pub use self::builder::Builder;
 
-use std::convert::TryFrom;
-
 use futures::{stream, Stream};
 use noodles_bgzf as bgzf;
 use noodles_core::Region;
-use noodles_csi::{BinningIndex, BinningIndexReferenceSequence};
+use noodles_csi::{binning_index::ReferenceSequenceExt, BinningIndex};
 use noodles_sam::header::{ReferenceSequence, ReferenceSequences};
 use tokio::io::{self, AsyncRead, AsyncReadExt, AsyncSeek};
 
-use self::query::query;
+use self::{query::query, record::read_record};
 use crate::{
     reader::{bytes_with_nul_to_string, resolve_region},
     Record, MAGIC_NUMBER,
@@ -301,15 +299,15 @@ where
     /// # Ok(())
     /// # }
     /// ```
-    pub fn query<I, IRS>(
+    pub fn query<I, RS>(
         &mut self,
         reference_sequences: &ReferenceSequences,
         index: &I,
         region: &Region,
     ) -> io::Result<impl Stream<Item = io::Result<Record>> + '_>
     where
-        I: BinningIndex<IRS>,
-        IRS: BinningIndexReferenceSequence,
+        I: BinningIndex<RS>,
+        RS: ReferenceSequenceExt,
     {
         let (reference_sequence_id, interval) = resolve_region(reference_sequences, region)?;
         let chunks = index.query(reference_sequence_id, interval)?;
@@ -389,22 +387,6 @@ where
     ReferenceSequence::new(name, l_ref).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
 }
 
-async fn read_record<R>(reader: &mut R, record: &mut Record) -> io::Result<usize>
-where
-    R: AsyncRead + Unpin,
-{
-    let block_size = match reader.read_u32_le().await {
-        Ok(bs) => usize::try_from(bs).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?,
-        Err(ref e) if e.kind() == io::ErrorKind::UnexpectedEof => return Ok(0),
-        Err(e) => return Err(e),
-    };
-
-    record.resize(block_size);
-    reader.read_exact(record).await?;
-
-    Ok(block_size)
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -458,7 +440,7 @@ mod tests {
         let mut reader = &data[..];
         let actual = read_reference_sequences(&mut reader).await?;
 
-        let expected: ReferenceSequences = vec![("sq0", 8)]
+        let expected: ReferenceSequences = [("sq0", 8)]
             .into_iter()
             .map(|(name, len)| ReferenceSequence::new(name, len).map(|rs| (name.into(), rs)))
             .collect::<Result<_, _>>()?;
@@ -467,21 +449,4 @@ mod tests {
 
         Ok(())
     }
-
-    #[tokio::test]
-    async fn test_read_record() -> io::Result<()> {
-        let data = [
-            0x04, 0x00, 0x00, 0x00, // block_size = 4
-            0x6e, 0x64, 0x6c, 0x73, // ...
-        ];
-
-        let mut reader = &data[..];
-        let mut record = Record::default();
-        let block_size = read_record(&mut reader, &mut record).await?;
-
-        assert_eq!(block_size, 4);
-        assert_eq!(&record[..], &data[4..]);
-
-        Ok(())
-    }
 }
