@@ -1,15 +1,15 @@
 mod builder;
 mod record;
+mod sam_record;
 
 pub use self::builder::Builder;
 
-use std::{convert::TryFrom, ffi::CString};
+use std::ffi::CString;
 
 use noodles_bgzf as bgzf;
 use noodles_sam as sam;
 use tokio::io::{self, AsyncWrite, AsyncWriteExt};
 
-use self::record::write_sam_record;
 use crate::Record;
 
 /// An async BAM writer.
@@ -139,7 +139,7 @@ where
     /// # }
     /// ```
     pub async fn write_record(&mut self, record: &Record) -> io::Result<()> {
-        write_record(&mut self.inner, record).await
+        record::write_record(&mut self.inner, record).await
     }
 
     /// Writes a SAM record.
@@ -167,7 +167,7 @@ where
         reference_sequences: &sam::header::ReferenceSequences,
         record: &sam::Record,
     ) -> io::Result<()> {
-        write_sam_record(&mut self.inner, reference_sequences, record).await
+        sam_record::write_sam_record(&mut self.inner, reference_sequences, record).await
     }
 }
 
@@ -230,19 +230,6 @@ where
     Ok(())
 }
 
-async fn write_record<W>(writer: &mut W, record: &Record) -> io::Result<()>
-where
-    W: AsyncWrite + Unpin,
-{
-    let block_size =
-        u32::try_from(record.len()).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-    writer.write_u32_le(block_size).await?;
-
-    writer.write_all(record).await?;
-
-    Ok(())
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
