@@ -1,6 +1,6 @@
 //! BAM reader and iterators.
 
-mod query;
+pub(crate) mod query;
 pub mod record;
 mod records;
 mod unmapped_records;
@@ -8,7 +8,6 @@ mod unmapped_records;
 pub use self::{query::Query, records::Records, unmapped_records::UnmappedRecords};
 
 use std::{
-    convert::TryFrom,
     ffi::CStr,
     io::{self, Read, Seek},
 };
@@ -16,9 +15,10 @@ use std::{
 use byteorder::{LittleEndian, ReadBytesExt};
 use noodles_bgzf as bgzf;
 use noodles_core::{region::Interval, Region};
-use noodles_csi::{BinningIndex, BinningIndexReferenceSequence};
+use noodles_csi::{binning_index::ReferenceSequenceExt, BinningIndex};
 use noodles_sam::header::{ReferenceSequence, ReferenceSequences};
 
+use self::record::read_record;
 use super::{bai, Record, MAGIC_NUMBER};
 
 /// A BAM reader.
@@ -55,6 +55,7 @@ where
     R: Read,
 {
     inner: bgzf::Reader<R>,
+    buf: Vec<u8>,
 }
 
 impl<R> Reader<R>
@@ -75,6 +76,7 @@ where
     pub fn new(reader: R) -> Self {
         Self {
             inner: bgzf::Reader::new(reader),
+            buf: Vec::new(),
         }
     }
 
@@ -157,7 +159,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn read_record(&mut self, record: &mut Record) -> io::Result<usize> {
-        read_record(&mut self.inner, record)
+        read_record(&mut self.inner, &mut self.buf, record)
     }
 
     /// Returns an iterator over records starting from the current stream position.
@@ -262,15 +264,15 @@ where
     /// }
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
-    pub fn query<I, IRS>(
+    pub fn query<I, RS>(
         &mut self,
         reference_sequences: &ReferenceSequences,
         index: &I,
         region: &Region,
     ) -> io::Result<Query<'_, R>>
     where
-        I: BinningIndex<IRS>,
-        IRS: BinningIndexReferenceSequence,
+        I: BinningIndex<RS>,
+        RS: ReferenceSequenceExt,
     {
         let (reference_sequence_id, interval) = resolve_region(reference_sequences, region)?;
         let chunks = index.query(reference_sequence_id, interval)?;
@@ -378,22 +380,6 @@ where
     ReferenceSequence::new(name, l_ref).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
 }
 
-fn read_record<R>(reader: &mut R, record: &mut Record) -> io::Result<usize>
-where
-    R: Read,
-{
-    let block_size = match reader.read_u32::<LittleEndian>() {
-        Ok(bs) => usize::try_from(bs).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?,
-        Err(ref e) if e.kind() == io::ErrorKind::UnexpectedEof => return Ok(0),
-        Err(e) => return Err(e),
-    };
-
-    record.resize(block_size);
-    reader.read_exact(record)?;
-
-    Ok(block_size)
-}
-
 pub(crate) fn bytes_with_nul_to_string(buf: &[u8]) -> io::Result<String> {
     CStr::from_bytes_with_nul(buf)
         .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
@@ -484,7 +470,7 @@ mod tests {
         let mut reader = &data[..];
         let actual = read_reference_sequences(&mut reader)?;
 
-        let expected: ReferenceSequences = vec![("sq0", 8)]
+        let expected: ReferenceSequences = [("sq0", 8)]
             .into_iter()
             .map(|(name, len)| ReferenceSequence::new(name, len).map(|rs| (name.into(), rs)))
             .collect::<Result<_, _>>()?;
@@ -493,21 +479,4 @@ mod tests {
 
         Ok(())
     }
-
-    #[test]
-    fn test_read_record() -> io::Result<()> {
-        let data = [
-            0x04, 0x00, 0x00, 0x00, // block_size = 4
-            0x6e, 0x64, 0x6c, 0x73, // ...
-        ];
-
-        let mut reader = &data[..];
-        let mut record = Record::default();
-        let block_size = read_record(&mut reader, &mut record)?;
-
-        assert_eq!(block_size, 4);
-        assert_eq!(&record[..], &data[4..]);
-
-        Ok(())
-    }
 }
