@@ -1,11 +1,11 @@
 //! BAM record CIGAR operation.
 
-use std::{convert::TryFrom, error, fmt, mem};
+use std::{error, fmt, mem};
 
 use byteorder::{ByteOrder, LittleEndian};
 use noodles_sam::record::cigar::op::Kind;
 
-const MAX_LENGTH: u32 = 268_435_455; // 2^28 - 1
+const MAX_LENGTH: u32 = (1 << 28) - 1;
 
 /// A BAM record CIGAR operation.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
@@ -173,6 +173,8 @@ impl fmt::Display for TryFromByteSliceError {
 impl TryFrom<&[u8]> for Op {
     type Error = TryFromByteSliceError;
 
+    #[allow(useless_deprecated)]
+    #[deprecated(since = "0.8.0", note = "Use `TryFrom<u32>` instead.")]
     fn try_from(buf: &[u8]) -> Result<Self, Self::Error> {
         if buf.len() < mem::size_of::<u32>() {
             return Err(TryFromByteSliceError::Invalid(buf.len()));
