@@ -1,7 +1,7 @@
-pub(crate) mod record;
+mod record;
+pub(crate) mod sam_record;
 
 use std::{
-    convert::TryFrom,
     ffi::CString,
     io::{self, Write},
 };
@@ -156,7 +156,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn write_record(&mut self, record: &Record) -> io::Result<()> {
-        write_record(&mut self.inner, record)
+        record::write_record(&mut self.inner, record)
     }
 
     /// Writes a SAM record.
@@ -180,7 +180,7 @@ where
         reference_sequences: &ReferenceSequences,
         record: &sam::Record,
     ) -> io::Result<()> {
-        record::write_sam_record(&mut self.inner, reference_sequences, record)
+        sam_record::write_sam_record(&mut self.inner, reference_sequences, record)
     }
 }
 
@@ -242,23 +242,8 @@ where
     Ok(())
 }
 
-fn write_record<W>(writer: &mut W, record: &Record) -> io::Result<()>
-where
-    W: Write,
-{
-    let block_size =
-        u32::try_from(record.len()).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-    writer.write_u32::<LittleEndian>(block_size)?;
-
-    writer.write_all(record)?;
-
-    Ok(())
-}
-
 #[cfg(test)]
 mod tests {
-    use std::convert::TryFrom;
-
     use noodles_sam::record::Data;
 
     use crate::{record::sequence::Base, Reader, Record};
@@ -289,7 +274,7 @@ mod tests {
 
     #[test]
     fn test_write_reference_sequences() -> Result<(), Box<dyn std::error::Error>> {
-        let reference_sequences = vec![("sq0", 8)]
+        let reference_sequences = [("sq0", 8)]
             .into_iter()
             .map(|(name, len)| ReferenceSequence::new(name, len).map(|rs| (name.into(), rs)))
             .collect::<Result<_, _>>()?;
@@ -309,22 +294,6 @@ mod tests {
         Ok(())
     }
 
-    #[test]
-    fn test_write_record() -> Result<(), Box<dyn std::error::Error>> {
-        let record = Record::default();
-
-        let mut buf = Vec::new();
-        write_record(&mut buf, &record)?;
-
-        let block_size = u32::try_from(record.len())?;
-        let mut expected = block_size.to_le_bytes().to_vec();
-        expected.extend_from_slice(&record);
-
-        assert_eq!(buf, expected);
-
-        Ok(())
-    }
-
     #[test]
     fn test_write_sam_record() -> Result<(), Box<dyn std::error::Error>> {
         let mut writer = Writer::new(Vec::new());
@@ -416,9 +385,7 @@ mod tests {
         let expected = [Base::A, Base::T, Base::C, Base::G];
         assert_eq!(actual, expected);
 
-        let actual = record.quality_scores();
-        let expected = [255, 255, 255, 255];
-        assert_eq!(*actual, expected);
+        assert_eq!(record.quality_scores().as_ref(), [255, 255, 255, 255]);
 
         Ok(())
     }
@@ -446,9 +413,7 @@ mod tests {
         let expected = [Base::A, Base::T, Base::C, Base::G];
         assert_eq!(actual, expected);
 
-        let actual = record.quality_scores();
-        let expected = [45, 35, 43, 50];
-        assert_eq!(*actual, expected);
+        assert_eq!(record.quality_scores().as_ref(), [45, 35, 43, 50]);
 
         Ok(())
     }
@@ -481,7 +446,7 @@ mod tests {
         reader.read_record(&mut record)?;
 
         let bam_data = record.data();
-        let mut fields = bam_data.fields();
+        let mut fields = bam_data.values();
 
         assert_eq!(
             fields.next().transpose()?,
